
-----------
--idTip
-----------

--
local hooksecurefunc, pairs, select, tonumber, tostring, UnitBuff, UnitDebuff, UnitAura, UnitIsPlayer, GetGlyphSocketInfo =
	  hooksecurefunc, pairs, select, tonumber, tostring, UnitBuff, UnitDebuff, UnitAura, UnitIsPlayer, GetGlyphSocketInfo;
--
local strformat = string.format;
local strfind = string.find;
local strsub = string.sub;
local tinsert = table.insert;
--]]

--------------
--spell owner
--[[
local _G = getfenv(0);
local _UnitAura = _G.UnitAura;
local _UnitName = _G.UnitName;
local _strformat = _G.string.format;
local _strsub = _G.string.sub;
--]]
local TOOLTIP_TEXT = "Applied by %s";
local PET_TEXT = "%s <%s>";
--------------------
--end spell owner]]
--------------------

--
local types = {

	--spell       = "SpellID",
	--item        = "ItemID",
	glyph       = "GlyphID",
	unit        = "NPC_ID",
	player		= "Player_ID",
	quest       = "QuestID",
	--talent      = "TalentID",
	achievement = "AchievementID"
};--]]

-----------------------------------
--custom ui replacements
-----------------------------------
--[[
BUFF_MIN_ALPHA = 1;
BUFF_DURATION_WARNING_TIME = 31;
--]]
tinsert(CHAT_FONT_HEIGHTS, 2, 13);

--[[
SECONDS_ABBR = "%d sec";
MINUTES_ABBR = "%d min";
HOURS_ABBR = "%d hr";
DAYS_ABBR = "%d day";
--]]
-----------------------------------
--end ui replacements
-----------------------------------

local function amd(text)
	DEFAULT_CHAT_FRAME:AddMessage(text);
end

-------------------------
-- slashcmd
--
local slash_tip = "/qwe";

SLASH_TIP1 = slash_tip;
local function handler(msg)
	local cmd = msg:lower();

	if type(tonumber(cmd:sub(1))) == "number" then

		if tonumber(cmd:sub(1)) then

			local spid = tonumber(cmd:sub(1));
			local name = GetSpellLink(spid);

			if name ~= nil then

				amd("spell link: "..GetSpellLink(spid));
			else

				amd("unknown spellID");
			end

		end

	else

		amd("only works with spellIDs");
	end
end
SlashCmdList["TIP"] = handler;
--]]
--end slashcmd
-----------------------

-----------------------
-- new output functions
--
local function _newAddLine(tooltip, id, type, noEmptyLine)

	if not noEmptyLine then tooltip:AddLine(" ") end;
	--tooltip:AddDoubleLine(type .. ":", "|cffffffff" .. id);
	tooltip:AddLine(type .. ": |cffffffff" .. id);
	tooltip:Show();
end

----------------------------------------------
-- All types, primarily for linked tooltips --
--
local function _newSetHyperlink(self, link)
	local type, id = string.match(link, "^(%a+):(%d+)");
	if not type or not id then return end;
	--if type == "spell" or type == "enchant" or type == "trade" then
		--_newAddLine(self, id, types.spell);
	--elseif type == "glyph" then
	if type == "glyph" then
		_newAddLine(self, id, types.glyph);
	--elseif type == "talent" then
		--_newAddLine(self, id, types.talent);
	elseif type == "quest" then
		_newAddLine(self, id, types.quest);
	elseif type == "achievement" then
		_newAddLine(self, id, types.achievement);
	--elseif type == "item" then
		--_newAddLine(self, id, types.item);
    end
end
--
-- end output functions
-----------------------
--
hooksecurefunc(ItemRefTooltip, "SetHyperlink", _newSetHyperlink)
hooksecurefunc(GameTooltip, "SetHyperlink", _newSetHyperlink)
--[[
-- Spells
hooksecurefunc(GameTooltip, "SetUnitBuff", function(self, ...)
	local id = select(11, UnitBuff(...));
	if id then _newAddLine(self, id, types.spell) end;
end)

hooksecurefunc(GameTooltip, "SetUnitDebuff", function(self,...)
	local id = select(11, UnitDebuff(...));
	if id then _newAddLine(self, id, types.spell) end;
end)

hooksecurefunc(GameTooltip, "SetUnitAura", function(self,...)
	local id = select(11, UnitAura(...));
	if id then _newAddLine(self, id, types.spell) end;
end)

hooksecurefunc("SetItemRef", function(link, ...)
	local id = tonumber(link:match("spell:(%d+)"));
	if id then _newAddLine(ItemRefTooltip, id, types.spell) end;
end)
--
GameTooltip:HookScript("OnTooltipSetSpell", function(self)
	local id = select(3, self:GetSpell());
	if id then _newAddLine(self, id, types.spell) end;
end)
--]]
-- new hooks --
---------------

---------------
-- Units IDs --
--
GameTooltip:HookScript("OnTooltipSetUnit", function(self)
	local unit = select(2, self:GetUnit());
	if unit then
		if not UnitIsPlayer(unit) then
			local id = tonumber(strsub(UnitGUID(unit) or "", -13, -9), 16);
			if id and id ~= 0 then _newAddLine(GameTooltip, id, types.unit) end;
		else
			local id = UnitGUID(unit);
			_newAddLine(GameTooltip, id, types.player);
		end
	end
end)
--]]
--end unit ids
----------------
--------------
--[[ Units IDs --obsolete
--------------
local f = CreateFrame("frame");
f:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
f:SetScript("OnEvent", function()
    if GameTooltip:IsVisible() and not UnitIsPlayer("mouseover") then
        local id = tonumber(UnitGUID("mouseover"):sub(-13, -9), 16);
        if id ~= 0 then
            _newAddLine(GameTooltip, id, types.unit);
        end
    end

	if GameTooltip:IsVisible() and UnitIsPlayer("mouseover") then
		local id = UnitGUID("mouseover");
		if id ~= 0 then
			_newAddLine(GameTooltip, id, "Player_ID");
		end
    end
end)
-----------------
--end unit ids
-----------------]]

-------------
-- itemIDs --
--[[
local function attachItemTooltip(self)
    local link = select(2, self:GetItem());
    if link then
        --local id = select(3, strfind(link, "^|%x+|Hitem:(%-?%d+):(%d+):(%d+).*"));
		local id = select(3, strfind(link, "^|%x+|Hitem:(%-?%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%-?%d+):(%-?%d+)"));
        if id then _newAddLine(self, id, types.item) end;
    end
end

GameTooltip:HookScript("OnTooltipSetItem", attachItemTooltip)
--ItemRefTooltip:HookScript("OnTooltipSetItem", attachItemTooltip)
ItemRefShoppingTooltip1:HookScript("OnTooltipSetItem", attachItemTooltip)
ItemRefShoppingTooltip2:HookScript("OnTooltipSetItem", attachItemTooltip)
ItemRefShoppingTooltip3:HookScript("OnTooltipSetItem", attachItemTooltip)
ShoppingTooltip1:HookScript("OnTooltipSetItem", attachItemTooltip)
ShoppingTooltip2:HookScript("OnTooltipSetItem", attachItemTooltip)
ShoppingTooltip3:HookScript("OnTooltipSetItem", attachItemTooltip)
--]]
-- end itemIDs --
-----------------

------------
-- Glyphs --
hooksecurefunc(GameTooltip, "SetGlyph", function(self, ...)
    local id = select(4, GetGlyphSocketInfo(...));
    if id then _newAddLine(self, id, types.glyph) end;
end)

hooksecurefunc(GameTooltip, "SetGlyphByID", function(self, id)
    if id then _newAddLine(self, id, types.glyph) end;
end)
-- end glyphs --
----------------

-------------------------------
-- Achievement Frame Tooltips
local f = CreateFrame("frame");
f:RegisterEvent("ADDON_LOADED");
f:SetScript("OnEvent", function(_, _, arg1)
    if arg1 == "Blizzard_AchievementUI" then
        for i, button in ipairs(AchievementFrameAchievementsContainer.buttons) do
            button:HookScript("OnEnter", function()
                GameTooltip:SetOwner(button, "ANCHOR_NONE");
                GameTooltip:SetPoint("TOPLEFT", button, "TOPRIGHT", 0, 0);
                _newAddLine(GameTooltip, button.id, types.achievement, true);
                GameTooltip:Show();
            end)
            button:HookScript("OnLeave", function()
                GameTooltip:Hide();
            end)
        end
    end
end)
--]]
-- end achievement frame --
---------------------------

----------------------
--digital time
--
local function timeformat(time)
	-- Nothing is nothing
	if time <= 0 then
		return "";
	end

	if time < 5 then
		return strformat("%d:%01d", time, time*10 % 10);
		--return strformat("%d:%02d", time, time*100 % 100);
	end

	if time < 60 then
	-- display anything < 1m as xs
		return strformat("%d s", time);
	end
	if time < 3660 then
	-- display < 1h as mm:ss
		return strformat("%d:%02d", time / 60, time % 60);
	end
	-- display anything over an hour in hh:mmh format (eg. 2:43h)
	return strformat("%d:%02dh", time / 3600, (time % 3600) / 60 );
end

--hooksecurefunc("AuraButton_UpdateDuration", function(auraButton, timeLeft) auraButton.duration:SetFormattedText(timeLeft) end)
hooksecurefunc("AuraButton_UpdateDuration", function(auraButton, timeLeft)
	function AuraButton_UpdateDuration(auraButton, timeLeft)
		local duration = auraButton.duration;

			if (SHOW_BUFF_DURATIONS == "1" and timeLeft) then

				duration:SetFormattedText(timeformat(timeLeft));

					if (timeLeft < BUFF_DURATION_WARNING_TIME) then

						--duration:SetVertexColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
						duration:SetVertexColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);

					else

						--duration:SetVertexColor(GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b);
						duration:SetVertexColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
					end

				duration:Show();

			else

				duration:Hide();
			end
	end
end)
--end digital time]]
----------------------


----------------------
-- original version --
--
local function addLine(self, id, isItem)
	if isItem then
		self:AddLine(" ")
		self:AddLine("ItemID: |cffffffff"..id)
		--self:AddDoubleLine("ItemID:", "|cffffffff"..id)
	else
		self:AddLine(" ")
		self:AddLine("SpellID: |cffffffff"..id)
		--self:AddDoubleLine("SpellID:", "|cffffffff"..id)
	end
	self:Show()
end

-----------------
-- Spell Hooks --
-----------------
hooksecurefunc(GameTooltip, "SetUnitBuff", function(self,...)
	local id = select(11, UnitBuff(...))
	if id then addLine(self, id) end
end)

hooksecurefunc(GameTooltip, "SetUnitDebuff", function(self,...)
	local id = select(11, UnitDebuff(...))
	if id then addLine(self, id) end
end)

hooksecurefunc(GameTooltip, "SetUnitAura", function(self,...)
	local id = select(11, UnitAura(...))
	if id then addLine(self, id) end
end)

GameTooltip:HookScript("OnTooltipSetSpell", function(self)
	local id = select(3, self:GetSpell())
	if id then addLine(self, id) end
end)

----------------
-- Item Hooks --
----------------
hooksecurefunc("SetItemRef", function(link, ...)
	local id = tonumber(link:match("spell:(%d+)"))
	if id then addLine(ItemRefTooltip, id) end
end)

local function attachItemTooltip(self)
	local link = select(2, self:GetItem())
	if not link then return end
	local id = select(3, strfind(link, "^|%x+|Hitem:(%-?%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%-?%d+):(%-?%d+)"))
	if id then addLine(self, id, true) end
end

GameTooltip:HookScript("OnTooltipSetItem", attachItemTooltip)
ItemRefTooltip:HookScript("OnTooltipSetItem", attachItemTooltip)
ItemRefShoppingTooltip1:HookScript("OnTooltipSetItem", attachItemTooltip)
ItemRefShoppingTooltip2:HookScript("OnTooltipSetItem", attachItemTooltip)
ItemRefShoppingTooltip3:HookScript("OnTooltipSetItem", attachItemTooltip)
ShoppingTooltip1:HookScript("OnTooltipSetItem", attachItemTooltip)
ShoppingTooltip2:HookScript("OnTooltipSetItem", attachItemTooltip)
ShoppingTooltip3:HookScript("OnTooltipSetItem", attachItemTooltip)
--]]
--end original
----------------

----------------
-- spell owner
--
local function _SetUnitAura(tooltip, unit, id, filter)
	local _, _, _, _, _, _, _, caster = UnitAura(unit, id, filter); --*
	if caster then
		local display_name;
		if caster == "vehicle" or caster == "pet" then
			display_name = strformat(PET_TEXT, UnitName(caster), UnitName("player")); --*
		elseif strsub(caster, 1, 8) == "partypet" and strsub(caster, -6) ~= "target" then --*
			local party_id = strsub(caster, 9); --*
			display_name = strformat(PET_TEXT, UnitName(caster), UnitName("party"..party_id)); --*
		elseif strsub(caster, 1, 7) == "raidpet" and strsub(caster, -6) ~= "target" then --*
			local raid_id = strsub(caster, 8); --*
			display_name = strformat(PET_TEXT, UnitName(caster), UnitName("raid"..raid_id)); --*
		else
			display_name = UnitName(caster); --*
		end

		tooltip:AddDoubleLine(strformat(TOOLTIP_TEXT, display_name)); --*
		tooltip:Show();
	end
end

local function _SetUnitBuff(tooltip, unit, id, filter)
	_SetUnitAura(tooltip, unit, id, "HELPFUL "..(filter or ""));
end

local function _SetUnitDebuff(tooltip, unit, id, filter)
	_SetUnitAura(tooltip, unit, id, "HARMFUL "..(filter or ""));
end

hooksecurefunc(GameTooltip, "SetUnitAura", _SetUnitAura)
hooksecurefunc(GameTooltip, "SetUnitBuff", _SetUnitBuff)
hooksecurefunc(GameTooltip, "SetUnitDebuff", _SetUnitDebuff)
--end spell owner]]
-------------------





